function NM4M_TrainBase(DIR,p,ALT_NAME)

%trains for the data in the folders in DIR Gaussian Mixture Models. The
%data contained in one folder is considered to belong to one class. All GMM
%are stores in a data structure that can be used for classification using
%the GMMBayes toolbox.
% Inputs:
% DIR       folder with the spectral base data
% p         parameter structure (for docu see NM4M_ParamInit.m)
% ALT_NAME  specifies the data list names and the name of the classifier to
%           be written
%
%
% Andre Holzapfel
% Master Thesis Implementation
% Signal Processing Laboratory
% University of Crete, June 2006
% Supervisor: Jannis Stylianou

%   Copyright (c) 2008 University of Crete.
% 
%   This file is part of Non-negative Matrix Factorization for Music (NM4M)
% 
%   NM4M is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License version 2 as
%   published by the Free Software Foundation.
% 
%   NM4M is distributed in the hope that it will be useful, but
%   WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%   General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with NM4M; if not, write to the Free Software
%   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
%   02110-1301 USA
% 
%   See the file "COPYING" for the text of the license.

p = NM4M_ParamInit(p);%set parameters to standard values if not set

if p.covar_type == 'diag'
    %set the options needed by netlab toolbox, used for a diag covariance
    %model, performance usually much worse than with full matrices even
    %when taking more distributions
    options = zeros(18,1);
    options(5) = 3;%get initial centres in kmeans from the data, could be improved by a splitting procedure
    options(1)  = 1;		% Prints out error values.
    options(14) = p.max_iter;		% Number of iterations.
    options(3) = 0.0001; %stop if there is no change in likelihood larger than that
end
TRAIN_LIST_NAME = [ALT_NAME '_TrainList.txt'];
TEST_LIST_NAME = [ALT_NAME '_TestList.txt'];

D = dir(DIR);k=0;
for i=1:length(D)
    if(D(i).isdir & ~(D(i).name(1) == '.'))
        k=k+1;
        soundDir(k)=D(i);
    end
end
if(k>0)
    NumDatAllClass = 0;
    numBases = zeros(1,k);
    %train separate GMM for each class
    for i=1:k
        ClassName{i} = fullfile( DIR, soundDir(i).name) ;
        fprintf('\nBuilding GMM model for: %s\n', ClassName{i});
        sfxfilename = fullfile( ClassName{i}, TRAIN_LIST_NAME );
        sfxfilename_test = fullfile( ClassName{i}, TEST_LIST_NAME );
        fileList = readsfxfiles(sfxfilename);
        fileList_test = readsfxfiles(sfxfilename_test);
        if isempty(fileList)
            fprintf('Skipping folder %s because it contains no files!\n',ClassName{i});
            continue;
        end
        W_all = [];%read the bases of all songs of a class into this structure
        baseperblock = [];%for printing out the avg number of base vectors per block
        for j=1:length(fileList)
            AllList{i}.file{j}=fullfile(ClassName{i},fileList{j});
            open(AllList{i}.file{j});
            W = cat(2,ans.W(1:length(ans.W)).data);
            baseperblock(j) = size(W,2) / length(ans.W);
            W_all = cat(2,W_all,W);
        end
        fprintf('The average number of bases per block is %i \n',mean(baseperblock));
        if p.decomp_meth == 2
            W_all = W_all - repmat(mean(W_all,2),1,size(W_all,2));
            C_xx = princomp(W_all');
            W_all = (W_all' * C_xx(:,1:20))';
        end
        %go for the gmm estimation
        fprintf('\n Starting EM ... \n');
        if (p.covar_type == 'full')
            [tmp,stat] = gmmb_em(W_all','maxloops',p.max_iter,'components',p.num_clusters,'init','kmeans','verbose',1);
            mu{i} = tmp.mu;
            sigma{i} = tmp.sigma;
            weight{i} = tmp.weight;
            prior(i) = size(W_all,2);
            stats{i} = stat;
        elseif (p.covar_type == 'diag')
            tmp = gmm(size(W_all,1),p.num_clusters,p.covar_type);
            options(14)=5;
            tmp = gmminit(tmp,W_all',options);
            options(14) = p.max_iter;
            tmp = gmmem(tmp,W_all',options);
            mu{i} = tmp.centres';
            for j = 1:p.num_clusters
                sigma{1,1}(:,:,j) = diag(tmp.covars(j,:)-eps)+eps;
            end
            weight{i} = tmp.priors;
            prior(i) = size(W_all,2);
        end
        NumDatAllClass = NumDatAllClass + size(W_all,2);
        numBases(i) = mean(baseperblock);
    end
    %prior = prior ./ NumDatAllClass;
    prior = ones(1,k) ./k;%use a flat prior, better for very uneven distributed training examples
    prior = num2cell(prior);
    bayesS = struct('mu', mu,...
        'sigma', sigma,...
        'apriories', prior,...
        'weight', weight);
    save(fullfile(DIR, ['bayesS_'  ALT_NAME '_' num2str(p.num_clusters) '_' num2str(p.crossrun) '.mat']),'bayesS');
    %save(fullfile(DIR, ['numBases_'  ALT_NAME '_' num2str(p.num_clusters) '_' num2str(p.crossrun) '.mat']),'numBases');
else
    fprintf('\nOoooops! There are no appropriate files.\n');
end